package com.mingdao.sample.service;

import com.baidu.aip.contentcensor.EImgType;
import com.baidu.aip.ocr.AipOcr;
import com.google.gson.Gson;
import com.tencentcloudapi.ocr.v20181119.models.*;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class SampleOcrBaiduService {

    public static final String WORDS_RESULT = "words_result";

    @org.springframework.beans.factory.annotation.Value("${ocr.secretId}")
    private String secretId;
    @org.springframework.beans.factory.annotation.Value("${ocr.secretKey}")
    private String secretKey;

    /***
     * 通用文字识别
     * @param imageUrl
     * @return
     */
    public GeneralBasicOCRResponse generalBasicOCR(String imageUrl) {
        // 初始化一个AipOcr
        AipOcr client = new AipOcr("appId", secretId, secretKey);

        // 调用接口
        JSONObject res = client.basicGeneralUrl(imageUrl, new HashMap());
        TextDetection[] textDetections = null;
        if (res.has(WORDS_RESULT)) {
            JSONArray wordsResult = res.getJSONArray(WORDS_RESULT);
            textDetections = new TextDetection[wordsResult.length()];
            for (int i = 0; i < wordsResult.length(); i++) {
                JSONObject words = wordsResult.getJSONObject(i);
                TextDetection textDetection = new TextDetection();
                textDetection.setDetectedText(words.getString(WORDS));
                textDetections[i] = textDetection;
            }
        }
        //tencentcloudapi GeneralBasicOCRResponse
        GeneralBasicOCRResponse resp = new GeneralBasicOCRResponse();
        resp.setTextDetections(textDetections);

        return resp;
    }


    public static final String WORDS = "words";
    /***
     * 身份证识别
     * @param imageUrl
     * @return IDCardOCRResponse
     */
    public IDCardOCRResponse idCardOCR(String imageUrl) {
        AipOcr client = new AipOcr("appId", secretId, secretKey);
        // 调用接口
        JSONObject res = client.idcardUrl(imageUrl, "front", new HashMap());
        JSONObject wordsResult = res.getJSONObject("words_result");
        //tencentcloudapi IDCardOCRResponse
        IDCardOCRResponse resp = new IDCardOCRResponse();
        resp.setName(wordsResult.getJSONObject("姓名").getString(WORDS));
        resp.setNation(wordsResult.getJSONObject("民族").getString(WORDS));
        resp.setAddress(wordsResult.getJSONObject("住址").getString(WORDS));
        resp.setIdNum(wordsResult.getJSONObject("公民身份号码").getString(WORDS));
        resp.setBirth(wordsResult.getJSONObject("出生").getString(WORDS));
        resp.setSex(wordsResult.getJSONObject("性别").getString(WORDS));
        resp.setAdvancedInfo("");
        resp.setAuthority("");
        resp.setRequestId("");
        resp.setValidDate("");
        return resp;
    }



    public static final Map<String, String> KEY_MAP = new HashMap();
    public static final Map<String, String> ITEM_KEY_MAP = new HashMap();

    private Gson gson = new Gson();

    static {
        KEY_MAP.put("InvoiceType", "发票消费类型");
        KEY_MAP.put("InvoiceTypeOrg", "发票名称");
        KEY_MAP.put("InvoiceCode", "发票代码");
        KEY_MAP.put("InvoiceNum", "发票号码");
        KEY_MAP.put("MachineCode", "机器编号");
        KEY_MAP.put("CheckCode", "校验码");
        KEY_MAP.put("InvoiceDate", "开票日期");
        KEY_MAP.put("PurchaserName", "购买方名称");
        KEY_MAP.put("PurchaserRegisterNum", "购买方识别号");
        KEY_MAP.put("PurchaserAddress", "购买方地址、电话");
        KEY_MAP.put("PurchaserBank", "购买方开户行及账号");
        KEY_MAP.put("Password", "密码区");
        KEY_MAP.put("SellerName", "销售方名称");
        KEY_MAP.put("SellerRegisterNum", "销售方识别号");
        KEY_MAP.put("SellerAddress", "销售方地址、电话");
        KEY_MAP.put("SellerBank", "销售方开户行及账号");
        KEY_MAP.put("TotalAmount", "合计金额");
        KEY_MAP.put("TotalTax", "合计税额");
        KEY_MAP.put("AmountInWords", "价税合计(大写)");
        KEY_MAP.put("AmountInFiguers", "小写金额");
        KEY_MAP.put("Payee", "收款人");
        KEY_MAP.put("Checker", "复核");
        KEY_MAP.put("NoteDrawer", "开票人");
        KEY_MAP.put("Remarks", "备注");

        ITEM_KEY_MAP.put("CommodityName", "Name");
        ITEM_KEY_MAP.put("CommodityType", "Spec");
        ITEM_KEY_MAP.put("CommodityUnit", "Unit");
        ITEM_KEY_MAP.put("CommodityNum", "Quantity");
        ITEM_KEY_MAP.put("CommodityPrice", "UnitPrice");
        ITEM_KEY_MAP.put("CommodityAmount", "AmountWithoutTax");
        ITEM_KEY_MAP.put("CommodityTaxRate", "TaxRate");
        ITEM_KEY_MAP.put("CommodityTax", "TaxAmount");

    }

    /***
     * 发票识别
     * @param imageUrl
     * @return VatInvoiceOCRResponse
     */
    public VatInvoiceOCRResponse vatInvoiceOCR(String imageUrl) {

        // 初始化一个AipOcr
        AipOcr client = new AipOcr("appId", secretId, secretKey);
        // 调用接口
        JSONObject res = client.vatInvoice(imageUrl, EImgType.URL, new HashMap());
        VatInvoiceOCRResponse resp = new VatInvoiceOCRResponse();
        List<TextVatInvoice> textVatInvoiceList = new ArrayList();
        JSONObject wordsResult = res.getJSONObject(WORDS_RESULT);

        for (String key : KEY_MAP.keySet()) {
            TextVatInvoice textVatInvoice = new TextVatInvoice();
            textVatInvoice.setName(KEY_MAP.get(key));
            textVatInvoice.setValue(wordsResult.getString(key));
            textVatInvoiceList.add(textVatInvoice);
        }
        TextVatInvoice[] textVatInvoices = new TextVatInvoice[textVatInvoiceList.size()];
        textVatInvoiceList.toArray(textVatInvoices);
        resp.setVatInvoiceInfos(textVatInvoices);

        List<VatInvoiceItem> items = new ArrayList();

        JSONArray nameArray = wordsResult.getJSONArray("CommodityName");
        for (int i = 0; i < nameArray.length(); i++) {
            Map<String, String> itemMap = new HashMap<>();
            int row=0;
            for (String key : ITEM_KEY_MAP.keySet()) {
                JSONArray array = wordsResult.getJSONArray(key);
                for (int j = 0; j < array.length(); j++) {
                    JSONObject item = array.getJSONObject(j);
                     row = item.getInt("row");
                    if (row == i + 1) {
                        String word = item.getString("word");
                        itemMap.put(ITEM_KEY_MAP.get(key), word);
                    }
                }
            }
            itemMap.put("LineNo", Integer.toString(row));
            itemMap.put("Spec", "");
            VatInvoiceItem vatInvoiceItem = gson.fromJson(gson.toJson(itemMap), VatInvoiceItem.class);
            items.add(vatInvoiceItem);
        }
        VatInvoiceItem[] items1 = new VatInvoiceItem[items.size()];
        items.toArray(items1);
        resp.setItems(items1);

        return resp;
    }

}
